@@ -18,37 +18,31 @@
 #============================================================================
 # Copyright (C) 2006 Ivan Kelly <ivan@ivankelly.net>
 #============================================================================
+# pylint: disable=I0011
 """
 Define the CLI interface for pwman3 and the helper functions
 """
-
 import pwman
 import pwman.exchange.importer as importer
 import pwman.exchange.exporter as exporter
 import pwman.util.generator as generator
 from pwman.data.nodes import Node
+from pwman.data.nodes import NewNode
 from pwman.data.tags import Tag
 from pwman.util.crypto import CryptoEngine
-#, CryptoBadKeyException, \
-#     CryptoPasswordMismatchException
-from pwman.util.callback import Callback
+from pwman.util.crypto import zerome
 import pwman.util.config as config
 import re
 import sys
 import os
-import struct
-import getpass
 import cmd
-#import traceback
 import time
 import select as uselect
-import subprocess as sp
 import ast
-
-if sys.platform != 'win32':
-    import tty
-    import termios
-    import fcntl
+from pwman.ui import tools
+from pwman.ui.tools import CliMenu
+from pwman.ui.tools import CliMenuItem
+from pwman.ui.tools import CLICallback
 
 try:
     import readline
@@ -56,28 +50,12 @@ try:
 except ImportError, e:
     _readline_available = False
 
-class CLICallback(Callback):
-    def getinput(self, question):
-        return raw_input(question)
-
-    def getsecret(self, question):
-        return getpass.getpass(question + ":")
-
-class ANSI(object):
-    Reset = 0
-    Bold = 1
-    Underscore = 2
-
-    Black = 30
-    Red = 31
-    Green = 32
-    Yellow = 33
-    Blue = 34
-    Magenta = 35
-    Cyan = 36
-    White = 37
 
+# pylint: disable=R0904
 class PwmanCli(cmd.Cmd):
+    """
+    UI class for MacOSX
+    """
     def error(self, exception):
         if (isinstance(exception, KeyboardInterrupt)):
             print
@@ -89,11 +67,12 @@ class PwmanCli(cmd.Cmd):
         return self.do_exit(args)
 
     def do_exit(self, args):
+        """exit the ui"""
         print
-        #try:
+        # try:
         #    print "goodbye"
         self._db.close()
-        #except DatabaseException, e:
+        # except DatabaseException, e:
         #    self.error(e)
         return True
 
@@ -103,7 +82,7 @@ class PwmanCli(cmd.Cmd):
         idstrs = args.split()
         for i in idstrs:
             m = rex.match(i)
-            if m == None:
+            if m is None:
                 try:
                     ids.append(int(i))
                 except ValueError:
@@ -116,44 +95,47 @@ class PwmanCli(cmd.Cmd):
         return ids
 
     def get_filesystem_path(self, default=""):
-        return getinput("Enter filename: ", default)
+        return tools.getinput("Enter filename: ", default)
 
     def get_username(self, default=""):
-        return getinput("Username: ", default)
+        return tools.getinput("Username: ", default)
 
-    def get_password(self, argsgiven, numerics=False,leetify=False, symbols=False,
-                     special_signs=False):
+    def get_password(self, argsgiven, numerics=False, leetify=False,
+                     symbols=False, special_signs=False):
         """
         in the config file:
         numerics -> numerics
         leetify -> symbols
         special_chars -> special_signs
-        """        
+        """
         if argsgiven == 1:
-            length = getinput("Password length (default 7): ", "7")
+            length = tools.getinput("Password length (default 7): ", "7")
             length = int(length)
-            (password, dumpme) = generator.generate_password(length, length, \
-                True, leetify, numerics, special_signs)
+            password, dumpme = generator.generate_password(length, length,
+                                                           True, leetify,
+                                                           numerics,
+                                                           special_signs)
             print "New password: %s" % (password)
             return password
         # no args given
-        password = getpassword("Password (Blank to generate): ", _defaultwidth, \
-            False)
+        password = tools.getpassword("Password (Blank to generate): ",
+                                     tools._defaultwidth, False)
         if len(password) == 0:
-            length = getinput("Password length (default 7): ", "7")
+            length = tools.getinput("Password length (default 7): ", "7")
             length = int(length)
-        
-            (password, dumpme) = generator.generate_password(length, length, \
-                True, leetify, numerics, special_signs)
+
+            (password, dumpme) = generator.generate_password(length, length,
+                                                             True, leetify,
+                                                             numerics,
+                                                             special_signs)
             print "New password: %s" % (password)
-        
         return password
-        
+
     def get_url(self, default=""):
-        return getinput("Url: ", default)
+        return tools.getinput("Url: ", default)
 
     def get_notes(self, default=""):
-        return getinput("Notes: ", default)
+        return tools.getinput("Notes: ", default)
 
     def get_tags(self, default=None):
         defaultstr = ''
@@ -180,7 +162,7 @@ class PwmanCli(cmd.Cmd):
                     else:
                         count += 1
 
-        taglist = getinput("Tags: ", defaultstr, complete)
+        taglist = tools.getinput("Tags: ", defaultstr, complete)
         tagstrings = taglist.split()
         tags = []
         for tn in tagstrings:
@@ -188,38 +170,28 @@ class PwmanCli(cmd.Cmd):
         return tags
 
     def print_node(self, node):
-        width = str(_defaultwidth)
+        width = str(tools._defaultwidth)
         print "Node %d." % (node.get_id())
-        print ("%"+width+"s %s") % (typeset("Username:", ANSI.Red),
+        print ("%"+width+"s %s") % (tools.typeset("Username:", tools.ANSI.Red),
                                     node.get_username())
-        print ("%"+width+"s %s") % (typeset("Password:", ANSI.Red),
+        print ("%"+width+"s %s") % (tools.typeset("Password:", tools.ANSI.Red),
                                     node.get_password())
-        print ("%"+width+"s %s") % (typeset("Url:", ANSI.Red),
+        print ("%"+width+"s %s") % (tools.typeset("Url:", tools.ANSI.Red),
                                     node.get_url())
-        print ("%"+width+"s %s") % (typeset("Notes:", ANSI.Red),
+        print ("%"+width+"s %s") % (tools.typeset("Notes:", tools.ANSI.Red),
                                     node.get_notes())
-        print typeset("Tags: ", ANSI.Red),
+        print tools.typeset("Tags: ", tools.ANSI.Red),
         for t in node.get_tags():
-            print " %s " % t.get_name(),
-        print
+            print " %s \n" % t.get_name(),
 
         def heardEnter():
-            i, o, e = uselect.select([sys.stdin], [], [], 0.0001)
-            for s in i:
-                if s == sys.stdin:
+            inpt, out, err = uselect.select([sys.stdin], [], [], 0.0001)
+            for stream in inpt:
+                if stream == sys.stdin:
                     sys.stdin.readline()
                     return True
                 return False
 
-        def heardEnterWin():
-            import msvcrt
-            c = msvcrt.kbhit()
-            if c == 1:
-                ret = msvcrt.getch()
-                if ret is not None:
-                    return True
-            return False
-
         def waituntil_enter(somepredicate, timeout, period=0.25):
             mustend = time.time() + timeout
             while time.time() < mustend:
@@ -228,20 +200,17 @@ class PwmanCli(cmd.Cmd):
                     break
                 time.sleep(period)
             self.do_cls('')
-        
+
         flushtimeout = int(config.get_value("Global", "cls_timeout"))
         if flushtimeout > 0:
-            if sys.platform != 'win32':
-                print "Type Enter to flush screen (autoflash in 5 sec.)"
-                waituntil_enter(heardEnter, flushtimeout)
-            else:
-                print "Press any key to flush screen (autoflash in 5 sec.)"
-                waituntil_enter(heardEnterWin, flushtimeout)
+            print "Type Enter to flush screen (autoflash in "\
+                  + "%d sec.)" % flushtimeout
+            waituntil_enter(heardEnter, flushtimeout)
 
     def do_tags(self, arg):
         tags = self._db.listtags()
         if len(tags) > 0:
-            tags[0].get_name() # hack to get password request before output
+            tags[0].get_name()  # hack to get password request before output
         print "Tags: ",
         if len(tags) == 0:
             print "None",
@@ -249,7 +218,6 @@ class PwmanCli(cmd.Cmd):
             print "%s " % (t.get_name()),
         print
 
-
     def complete_filter(self, text, line, begidx, endidx):
         strings = []
         enc = CryptoEngine.get()
@@ -263,7 +231,6 @@ class PwmanCli(cmd.Cmd):
                 strings.append(t.get_name())
         return strings
 
-
     def do_filter(self, args):
         tagstrings = args.split()
 
@@ -315,26 +282,26 @@ class PwmanCli(cmd.Cmd):
                 menu.add(CliMenuItem("Tags", self.get_tags,
                                      node.get_tags,
                                      node.set_tags))
-
                 menu.run()
                 self._db.editnode(i, node)
+                # when done with node erase it
+                zerome(node._password)
             except Exception, e:
                 self.error(e)
 
-
     def do_import(self, arg):
         try:
             args = arg.split()
             if len(args) == 0:
                 types = importer.Importer.types()
-                intype = select("Select filetype:", types)
+                intype = tools.select("Select filetype:", types)
                 imp = importer.Importer.get(intype)
-                infile = getinput("Select file:")
+                infile = tools.getinput("Select file:")
                 imp.import_data(self._db, infile)
             else:
                 for i in args:
                     types = importer.Importer.types()
-                    intype = select("Select filetype:", types)
+                    intype = tools.select("Select filetype:", types)
                     imp = importer.Importer.get(intype)
                     imp.import_data(self._db, i)
         except Exception, e:
@@ -345,11 +312,11 @@ class PwmanCli(cmd.Cmd):
             nodes = self.get_ids(arg)
 
             types = exporter.Exporter.types()
-            ftype = select("Select filetype:", types)
+            ftype = tools.select("Select filetype:", types)
             exp = exporter.Exporter.get(ftype)
-            out_file = getinput("Select output file:")
+            out_file = tools.getinput("Select output file:")
             if len(nodes) > 0:
-                b = getyesno("Export nodes %s?" % (nodes), True)
+                b = tools.getyesno("Export nodes %s?" % (nodes), True)
                 if not b:
                     return
                 exp.export_data(self._db, out_file, nodes)
@@ -361,8 +328,7 @@ class PwmanCli(cmd.Cmd):
                     tagstr = " for "
                     for t in tags:
                         tagstr += "'%s' " % (t.get_name())
-
-                b = getyesno("Export all nodes%s?" % (tagstr), True)
+                b = tools.getyesno("Export all nodes%s?" % (tagstr), True)
                 if not b:
                     return
                 exp.export_data(self._db, out_file, nodes)
@@ -384,8 +350,8 @@ class PwmanCli(cmd.Cmd):
         Password (Blank to generate):
         """
         errmsg = """could not parse config override, please input some"""\
-                 +""" kind of dictionary, e.g.: n {'leetify':False, """\
-                 +"""'numerics':True, 'special_chars':True}"""
+                 + """ kind of dictionary, e.g.: n {'leetify':False, """\
+                 + """'numerics':True, 'special_chars':True}"""
         try:
             username = self.get_username()
             if args:
@@ -397,10 +363,14 @@ class PwmanCli(cmd.Cmd):
                     raise Exception(errmsg)
                 password = self.get_password(1, **args)
             else:
-                numerics = config.get_value("Generator", "numerics").lower() == 'true'
+                numerics = config.get_value("Generator",
+                                            "numerics").lower() == 'true'
                 # TODO: allow custom leetifying through the config
-                leetify = config.get_value("Generator", "leetify").lower() == 'true' 
-                special_chars = config.get_value("Generator", "special_chars").lower() == 'true' 
+                leetify = config.get_value("Generator",
+                                           "leetify").lower() == 'true'
+                special_chars = config.get_value("Generator",
+                                                 "special_chars").lower() == \
+                    'true'
                 password = self.get_password(0,
                                              numerics=numerics,
                                              symbols=leetify,
@@ -434,9 +404,9 @@ class PwmanCli(cmd.Cmd):
         try:
             nodes = self._db.getnodes(ids)
             for n in nodes:
-                b = getyesno("Are you sure you want to delete '%s@%s'?"
-                              % (n.get_username(), n.get_url()), False)
-                if b == True:
+                b = tools.getyesno("Are you sure you want to delete '%s@%s'?"
+                                   % (n.get_username(), n.get_url()), False)
+                if b is True:
                     self._db.removenodes([n])
                     print "%s@%s deleted" % (n.get_username(), n.get_url())
         except Exception, e:
@@ -455,9 +425,9 @@ class PwmanCli(cmd.Cmd):
             self.do_filter(args)
         try:
             if sys.platform != 'win32':
-                rows, cols = gettermsize()
+                rows, cols = tools.gettermsize()
             else:
-                rows, cols = 18, 80 # fix this !
+                rows, cols = 18, 80
             nodeids = self._db.listnodes()
             nodes = self._db.getnodes(nodeids)
             cols -= 8
@@ -482,12 +452,13 @@ class PwmanCli(cmd.Cmd):
                     tagstring = tagstring[:tagstring_len-3] + "..."
 
                 fmt = "%%5d. %%-%ds %%-%ds" % (name_len, tagstring_len)
-                print typeset(fmt % (n.get_id(), name, tagstring),
-                              ANSI.Yellow, False)
+                print tools.typeset(fmt % (n.get_id(), name, tagstring),
+                                    tools.ANSI.Yellow, False)
                 i += 1
                 if i > rows-2:
                     i = 0
-                    c = getonechar("Press <Space> for more, or 'Q' to cancel")
+                    c = tools.getonechar("Press <Space> for more, "
+                                         "or 'Q' to cancel")
                     if c == 'q':
                         break
 
@@ -559,11 +530,13 @@ class PwmanCli(cmd.Cmd):
                 return None
             try:
                 node = self._db.getnodes(ids)
-                text_to_clipboards(node[0].get_password())
-                print "copied password for %s@%s clipboard... erasing in 10 sec..." % \
-                (node[0].get_username(), node[0].get_url())
+                tools.text_to_clipboards(node[0].get_password())
+                print "copied password for {}@{} clipboard".format(
+                    node[0].get_username(), node[0].get_url())
+
+                print "erasing in 10 sec..."
                 time.sleep(10)
-                text_to_clipboards("")
+                tools.text_to_clipboards("")
             except Exception, e:
                 self.error(e)
         else:
@@ -574,13 +547,16 @@ class PwmanCli(cmd.Cmd):
 
     def do_open(self, args):
         ids = self.get_ids(args)
+        if not args:
+            self.help_open()
+            return
         if len(ids) > 1:
             print "Can open only 1 link at a time ..."
             return None
         try:
             node = self._db.getnodes(ids)
             url = node[0].get_url()
-            open_url(url)
+            tools.open_url(url)
         except Exception, e:
             self.error(e)
 
@@ -589,13 +565,14 @@ class PwmanCli(cmd.Cmd):
     ##
     ## Help functions
     ##
+
     def usage(self, string):
         print "Usage: %s" % (string)
 
     def help_open(self):
         self.usage("open <ID>")
-        print "Launch default browser with 'xdg-open url',\n\
-the url must contain http:// or https://."
+        print "Launch default browser with 'xdg-open url',\n" \
+              + "the url must contain http:// or https://."
 
     def help_o(self):
         self.help_open()
@@ -613,7 +590,8 @@ the url must contain http:// or https://."
 
     def help_list(self):
         self.usage("list <tag> ...")
-        print "List nodes that match current or specified filter. ls is an alias."
+        print "List nodes that match current or specified filter." \
+              + " l is an alias."
 
     def help_EOF(self):
         self.help_exit()
@@ -653,13 +631,15 @@ the url must contain http:// or https://."
 
     def help_export(self):
         self.usage("export <ID|tag> ... ")
-        print "Exports a list of ids to an external format. If no IDs or tags are specified, then all nodes under the current filter are exported."
+        print "Exports a list of ids to an external format. If no IDs or" \
+              + " tags are specified, then all nodes under the current" \
+              + " filter are exported."
         self._mult_id_help()
 
     def help_new(self):
         self.usage("new")
-        print """Creates a new node., 
-You can override default config settings the following way:      
+        print """Creates a new node.,
+You can override default config settings the following way:
 pwman> n {'leetify':False, 'numerics':True}"""
 
     def help_rm(self):
@@ -671,7 +651,12 @@ pwman> n {'leetify':False, 'numerics':True}"""
         self._mult_id_help()
 
     def _mult_id_help(self):
-        print "Multiple ids and nodes can be specified, separated by a space. A range of ids can be specified in the format n-N. e.g. '10-20' would specify all nodes having ids from 10 to 20 inclusive. Tags are considered one-by-one. e.g. 'foo 2 bar' would yield to all nodes with tag 'foo', node 2 and all nodes with tag 'bar'."
+        print "Multiple ids and nodes can be specified, separated by a space."\
+              + "A range of ids can be specified in the format n-N. e.g. " \
+              + "'10-20' would specify all nodes having ids from 10 to 20 " \
+              + "inclusive. Tags are considered one-by-one. e.g. 'foo 2 bar'" \
+              + " would yield to all nodes with tag 'foo', node 2 and all "\
+              + "nodes with tag 'bar'."
 
     def help_exit(self):
         self.usage("exit")
@@ -679,11 +664,17 @@ pwman> n {'leetify':False, 'numerics':True}"""
 
     def help_save(self):
         self.usage("save [filename]")
-        print "Saves the current configuration to [filename]. If no filename is given, the configuration is saved to the file from which the initial configuration was loaded."
+        print "Saves the current configuration to [filename]. If no filename "\
+              + "is given, the configuration is saved to the file from which "\
+              + "the initial configuration was loaded."
 
     def help_set(self):
         self.usage("set [configoption] [value]")
-        print "Sets a configuration option. If no value is specified, the current value for [configoption] is output. If neither [configoption] nor [value] are specified, the whole current configuration is output. [configoption] must be of the format <section>.<option>"
+        print "Sets a configuration option. If no value is specified, the "\
+              + "current value for [configoption] is output. If neither "\
+              + "[configoption] nor [value] are specified, the whole current "\
+              + "configuration is output. [configoption] must be of the "\
+              + "format <section>.<option>"
 
     def help_passwd(self):
         self.usage("passwd")
@@ -691,7 +682,8 @@ pwman> n {'leetify':False, 'numerics':True}"""
 
     def help_forget(self):
         self.usage("forget")
-        print "Forgets the database password. Your password will need to be reentered before accessing the database again."
+        print "Forgets the database password. Your password will need to " \
+              + "be reentered before accessing the database again."
 
     def help_clear(self):
         self.usage("clear")
@@ -699,7 +691,8 @@ pwman> n {'leetify':False, 'numerics':True}"""
 
     def help_filter(self):
         self.usage("filter <tag> ...")
-        print "Filters nodes on tag. Arguments can be zero or more tags. Displays current tags if called without arguments."
+        print "Filters nodes on tag. Arguments can be zero or more tags. " \
+              + "Displays current tags if called without arguments."
 
     def help_tags(self):
         self.usage("tags")
@@ -711,6 +704,40 @@ pwman> n {'leetify':False, 'numerics':True}"""
         except Exception:
             pass
 
+    def __init__(self, db, hasxsel):
+        """
+        initialize CLI interface, set up the DB
+        connecion, see if we have xsel ...
+        """
+        _dbwarning = "\n*** WARNNING: You are using the old db format which" \
+                     + " uses cPickle, please upgrade your db !!! ***"
+        cmd.Cmd.__init__(self)
+        self.intro = "%s %s (c) visit: %s %s" % (pwman.appname, pwman.version,
+                                                 pwman.website, _dbwarning)
+        self._historyfile = config.get_value("Readline", "history")
+        self.hasxsel = hasxsel
+        try:
+            enc = CryptoEngine.get()
+            enc.set_callback(CLICallback())
+            self._db = db
+            self._db.open()
+        except Exception, e:
+            self.error(e)
+            sys.exit(1)
+        try:
+            readline.read_history_file(self._historyfile)
+        except IOError, e:
+            pass
+        self.prompt = "!pwman> "
+
+
+class PwmanCliNew(PwmanCli):
+    """
+    inherit from the old class, override
+    all the methods related to tags, and
+    newer Node format, so backward compatability is kept...
+    """
+
     def __init__(self, db, hasxsel):
         """
         initialize CLI interface, set up the DB
@@ -718,7 +745,7 @@ pwman> n {'leetify':False, 'numerics':True}"""
         """
         cmd.Cmd.__init__(self)
         self.intro = "%s %s (c) visit: %s" % (pwman.appname, pwman.version,
-                                            pwman.website)
+                                              pwman.website)
         self._historyfile = config.get_value("Readline", "history")
         self.hasxsel = hasxsel
         try:
@@ -732,260 +759,213 @@ pwman> n {'leetify':False, 'numerics':True}"""
 
         try:
             readline.read_history_file(self._historyfile)
-        except IOError, e:
+        except NameError, e:
             pass
 
         self.prompt = "pwman> "
 
+    def print_node(self, node):
+        width = str(tools._defaultwidth)
+        print "Node %d." % (node.get_id())
+        print ("%"+width+"s %s") % (tools.typeset("Username:", tools.ANSI.Red),
+                                    node.get_username())
+        print ("%"+width+"s %s") % (tools.typeset("Password:", tools.ANSI.Red),
+                                    node.get_password())
+        print ("%"+width+"s %s") % (tools.typeset("Url:", tools.ANSI.Red),
+                                    node.get_url())
+        print ("%"+width+"s %s") % (tools.typeset("Notes:", tools.ANSI.Red),
+                                    node.get_notes())
+        print tools.typeset("Tags: ", tools.ANSI.Red),
+        for t in node.get_tags():
+            print " %s " % t
+        print
 
+        def heardEnter():
+            i, o, e = uselect.select([sys.stdin], [], [], 0.0001)
+            for s in i:
+                if s == sys.stdin:
+                    sys.stdin.readline()
+                    return True
+                return False
 
-class PwmanCliMac(PwmanCli):
-    """
-    inherit from PwmanCli, override the right functions...
-    """
-    def do_copy(self, args):
-        ids = self.get_ids(args)
-        if len(ids) > 1:
-            print "Can only 1 password at a time..."
-        try:
-            node = self._db.getnodes(ids)
-            node[0].get_password()
-            text_to_mcclipboard(node[0].get_password())
-            print "copied password for %s@%s clipboard... erasing in 10 sec..." % \
-            (node[0].get_username(), node[0].get_url())
-            time.sleep(10)
-            text_to_clipboards("")
-        except Exception, e:
-            self.error(e)
-
-    def do_cp(self, args):
-        self.do_copy(args)
-
-    def do_open(self, args):
-        ids = self.get_ids(args)
-        if len(ids) > 1:
-            print "Can open only 1 link at a time ..."
-            return None
-        try:
-            node = self._db.getnodes(ids)
-            url = node[0].get_url()
-            open_url(url, macosx=True)
-        except Exception, e:
-            self.error(e)
-
-    def do_o(self, args):
-        self.do_open(args)
-
-    ##
-    ## Help functions
-    ##
-    def help_open(self):
-        self.usage("open <ID>")
-        print "Launch default browser with 'open url',\n\
-the url must contain http:// or https://."
-
-    def help_o(self):
-        self.help_open()
-
-    def help_copy(self):
-        self.usage("copy <ID>")
-        print "Copy password to Cocoa clipboard using pbcopy)"
-
-    def help_cp(self):
-        self.help_copy()
-
-_defaultwidth = 10
-
-def getonechar(question, width=_defaultwidth):
-    question = "%s " % (question)
-    print question.ljust(width),
-    sys.stdout.flush()
-
-    fd = sys.stdin.fileno()
-    tty_mode = tty.tcgetattr(fd)
-    tty.setcbreak(fd)
-    try:
-        ch = os.read(fd, 1)
-    finally:
-        tty.tcsetattr(fd, tty.TCSAFLUSH, tty_mode)
-    print ch
-    return ch
-
-def getyesno(question, defaultyes=False, width=_defaultwidth):
-    if (defaultyes):
-        default = "[Y/n]"
-    else:
-        default = "[y/N]"
-    ch = getonechar("%s %s" % (question, default), width)
-    if (ch == '\n'):
-        if (defaultyes):
-            return True
-        else:
-            return False
-    elif (ch == 'y' or ch == 'Y'):
-        return True
-    elif (ch == 'n' or ch == 'N'):
-        return False
-    else:
-        return getyesno(question, defaultyes, width)
-
-def gettermsize():
-    s = struct.pack("HHHH", 0, 0, 0, 0)
-    f = sys.stdout.fileno()
-    x = fcntl.ioctl(f, termios.TIOCGWINSZ, s)
-    rows, cols, width, height = struct.unpack("HHHH", x)
-    return rows, cols
-
-def getinput(question, default="", completer=None, width=_defaultwidth):
-    if (not _readline_available):
-        return raw_input(question.ljust(width))
-    else:
-        def defaulter(): 
-            """define default behavior startup"""
-            readline.insert_text(default)
-
-        readline.set_startup_hook(defaulter)
-        oldcompleter = readline.get_completer()
-        readline.set_completer(completer)
-
-        x = raw_input(question.ljust(width))
-
-        readline.set_completer(oldcompleter)
-        readline.set_startup_hook()
-        return x
-
-def getpassword(question, width=_defaultwidth, echo=False):
-    if echo:
-        print question.ljust(width),
-        return sys.stdin.readline().rstrip()
-    else:
-        while 1:
-            a1 = getpass.getpass(question.ljust(width))
-            if len(a1) == 0:
-                return a1
-            a2 = getpass.getpass("[Repeat] %s" % (question.ljust(width)))
-            if a1 == a2:
-                return a1
-            else:
-                print "Passwords don't match. Try again."
-
-
-def typeset(text, color, bold=False, underline=False):
-    if not config.get_value("Global", "colors") == 'yes':
-        return text
-    if (bold):
-        bold = "%d ;" % (ANSI.Bold)
-    else:
-        bold = ""
-    if (underline):
-        underline = "%d;" % (ANSI.Underscore)
-    else:
-        underline = ""
-    return "\033[%s%s%sm%s\033[%sm" % (bold, underline, color,
-                                       text, ANSI.Reset)
-
-def select(question, possible):
-    for i in range(0, len(possible)):
-        print ("%d - %-"+str(_defaultwidth)+"s") % (i+1, possible[i])
-    while 1:
-        uinput = getonechar(question)
-        if uinput.isdigit() and int(uinput) in range(1, len(possible)+1):
-            return possible[int(uinput)-1]
-
-def text_to_clipboards(text):
-    """
-    copy text to clipboard
-    credit:
-    https://pythonadventures.wordpress.com/tag/xclip/
-    """
-    # "primary":
-    try:
-        xsel_proc = sp.Popen(['xsel', '-pi'], stdin=sp.PIPE)
-        xsel_proc.communicate(text)
-        # "clipboard":
-        xsel_proc = sp.Popen(['xsel', '-bi'], stdin=sp.PIPE)
-        xsel_proc.communicate(text)
-    except OSError, e:
-        print e, "\nExecuting xsel failed, is it installed ?\n \
-please check your configuration file ... "
-
-
-def text_to_mcclipboard(text):
-    """
-    copy text to mac os x clip board
-    credit:
-    https://pythonadventures.wordpress.com/tag/xclip/
-    """
-    # "primary":
-    try:
-        pbcopy_proc = sp.Popen(['pbcopy'], stdin=sp.PIPE)
-        pbcopy_proc.communicate(text)
-    except OSError, e:
-        print e, "\nExecuting pbcoy failed..."
-
-
-def open_url(link, macosx=False):
-    """
-    launch xdg-open or open in MacOSX with url
-    """
-    uopen = "xdg-open"
-    if macosx:
-        uopen = "open"
-    try:
-        sp.Popen([uopen, link], stdin=sp.PIPE)
-    except OSError, e:
-        print "Executing open_url failed with:\n", e
-
-
-class CliMenu(object):
-    def __init__(self):
-        self.items = []
-
-    def add(self, item):
-        if (isinstance(item, CliMenuItem)):
-            self.items.append(item)
-        else:
-            print item.__class__
-
-    def run(self):
-        while True:
-            i = 0
-            for x in self.items:
-                i = i + 1
-                current = x.getter()
-                currentstr = ''
-                if type(current) == list:
-                    for c in current:
-                        currentstr += ("%s " % (c))
-                else:
-                    currentstr = current
-
-                print ("%d - %-"+str(_defaultwidth)+"s %s") % (i, x.name+":",
-                                                               currentstr)
-            print "%c - Finish editing" % ('X')
-            option = getonechar("Enter your choice:")
-            try:
-                # substract 1 because array subscripts start at 0
-                selection = int(option) - 1
-                print "selection, ", selection
-                # new value is created by calling the editor with the 
-                # previous value as a parameter 
-                # TODO: enable overriding password policy as if new node
-                # is created.
-                if selection == 1: # for password
-                    value = self.items[selection].editor(0)
-                else:
-                    value = self.items[selection].editor(self.items[selection].getter())
-                
-                self.items[selection].setter(value)
-            except (ValueError, IndexError):
-                if (option.upper() == 'X'):
+        def waituntil_enter(somepredicate, timeout, period=0.25):
+            mustend = time.time() + timeout
+            while time.time() < mustend:
+                cond = somepredicate()
+                if cond:
                     break
-                print "Invalid selection"
+                time.sleep(period)
+            self.do_cls('')
 
-class CliMenuItem(object):
-    def __init__(self, name, editor, getter, setter):
-        self.name = name
-        self.editor = editor
-        self.getter = getter
-        self.setter = setter
+        flushtimeout = int(config.get_value("Global", "cls_timeout"))
+        if flushtimeout > 0:
+            print "Type Enter to flush screen (autoflash in "\
+                  + "%d sec.)" % flushtimeout
+            waituntil_enter(heardEnter, flushtimeout)
+
+    def do_tags(self, arg):
+        tags = self._db.listtags()
+        # request password
+        enc = CryptoEngine.get()
+        if not enc.alive():
+            enc._getcipher()
+        print "Tags: ",
+        if len(tags) == 0:
+            print "None",
+        for t in tags:
+            print "%s " % t,
+        print
+
+    def get_tags(self, default=None):
+        defaultstr = ''
+
+        if default:
+            for t in default:
+                defaultstr += "%s " % (t)
+        else:
+            tags = self._db.currenttags()
+            for t in tags:
+                defaultstr += "%s " % (t)
+
+        strings = []
+        tags = self._db.listtags(True)
+        for t in tags:
+            # strings.append(t.get_name())
+            strings.append(t)
+
+        def complete(text, state):
+            count = 0
+            for s in strings:
+                if s.startswith(text):
+                    if count == state:
+                        return s
+                    else:
+                        count += 1
+
+        taglist = tools.getinput("Tags: ", defaultstr, complete)
+        tagstrings = taglist.split()
+        tags = []
+        for tn in tagstrings:
+            _Tag = Tag(tn)
+            tags.append(_Tag)
+        return tags
+
+    def do_list(self, args):
+        if len(args.split()) > 0:
+            self.do_clear('')
+            self.do_filter(args)
+        try:
+            if sys.platform != 'win32':
+                rows, cols = tools.gettermsize()
+            else:
+                rows, cols = 18, 80  # fix this !
+            nodeids = self._db.listnodes()
+            nodes = self._db.getnodes(nodeids)
+            cols -= 8
+            i = 0
+            for n in nodes:
+                tags = n.get_tags()
+                tags = filter(None, tags)
+                tagstring = ''
+                first = True
+                for t in tags:
+                    if not first:
+                        tagstring += ", "
+                    else:
+                        first = False
+                    tagstring += t
+
+                name = "%s@%s" % (n.get_username(), n.get_url())
+
+                name_len = cols * 2 / 3
+                tagstring_len = cols / 3
+                if len(name) > name_len:
+                    name = name[:name_len-3] + "..."
+                if len(tagstring) > tagstring_len:
+                    tagstring = tagstring[:tagstring_len-3] + "..."
+                fmt = "%%5d. %%-%ds %%-%ds" % (name_len, tagstring_len)
+                formatted_entry = tools.typeset(fmt % (n.get_id(),
+                                                name, tagstring),
+                                                tools.ANSI.Yellow, False)
+                print formatted_entry
+                i += 1
+                if i > rows-2:
+                    i = 0
+                    c = tools.getonechar("Press <Space> for more,"
+                                         " or 'Q' to cancel")
+                    if c == 'q':
+                        break
+
+        except Exception, e:
+            self.error(e)
+
+    def do_filter(self, args):
+        tagstrings = args.split()
+        try:
+            tags = []
+            for ts in tagstrings:
+                tags.append(Tag(ts))
+            self._db.filter(tags)
+            tags = self._db.currenttags()
+            print "Current tags: ",
+            if len(tags) == 0:
+                print "None",
+            for t in tags:
+                print "%s " % (t.get_name()),
+            print
+        except Exception, e:
+            self.error(e)
+
+    def do_new(self, args):
+        """
+        can override default config settings the following way:
+        Pwman3 0.2.1 (c) visit: http://github.com/pwman3/pwman3
+        pwman> n {'leetify':False, 'numerics':True, 'special_chars':True}
+        Password (Blank to generate):
+        """
+        errmsg = """could not parse config override, please input some"""\
+                 + """ kind of dictionary, e.g.: n {'leetify':False, """\
+                 + """'numerics':True, 'special_chars':True}"""
+        try:
+            username = self.get_username()
+            if args:
+                try:
+                    args = ast.literal_eval(args)
+                except Exception:
+                    raise Exception(errmsg)
+                if not isinstance(args, dict):
+                    raise Exception(errmsg)
+                password = self.get_password(1, **args)
+            else:
+                numerics = config.get_value(
+                    "Generator", "numerics").lower() == 'true'
+                # TODO: allow custom leetifying through the config
+                leetify = config.get_value(
+                    "Generator", "leetify").lower() == 'true'
+                special_chars = config.get_value(
+                    "Generator", "special_chars").lower() == 'true'
+                password = self.get_password(0,
+                                             numerics=numerics,
+                                             symbols=leetify,
+                                             special_signs=special_chars)
+            url = self.get_url()
+            notes = self.get_notes()
+            node = NewNode(username, password, url, notes)
+            tags = self.get_tags()
+            node.set_tags(tags)
+            self._db.addnodes([node])
+            print "Password ID: %d" % (node.get_id())
+            # when done with node erase it
+            zerome(password)
+        except Exception, e:
+            self.error(e)
+
+    def do_print(self, arg):
+        for i in self.get_ids(arg):
+            try:
+                node = self._db.getnodes([i])
+                self.print_node(node[0])
+                # when done with node erase it
+                zerome(node[0]._password)
+            except Exception, e:
+                self.error(e)
